// Copyright 2011 Julian Phillips.  All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package py

// #include "utils.h"
//import "C"

import "unsafe"

//export GoClassMapLen
func GoClassMapLen(obj unsafe.Pointer) int64 {
	// Get the class context
	ctxt := getClassContext(obj)

	// Turn the function into something we can call
	f := (*func(unsafe.Pointer) int64)(unsafe.Pointer(&ctxt.mp_len))

	return int64((*f)(obj))
}

//export GoClassMapGet
func GoClassMapGet(obj, arg unsafe.Pointer) unsafe.Pointer {
	// Get the class context
	ctxt := getClassContext(obj)

	// Turn the function into something we can call
	f := (*func(unsafe.Pointer, Object) (Object, error))(unsafe.Pointer(&ctxt.mp_get))

	key := newObject((*_PyObject)(arg))

	ret, err := (*f)(obj, key)
	if err != nil {
		raise(err)
		return nil
	}

	return unsafe.Pointer(c(ret))
}

//export GoClassMapSet
func GoClassMapSet(obj, arg1, arg2 unsafe.Pointer) int {
	// Get the class context
	ctxt := getClassContext(obj)

	// Turn the function into something we can call
	f := (*func(unsafe.Pointer, Object, Object) error)(unsafe.Pointer(&ctxt.mp_set))

	key := newObject((*_PyObject)(arg1))
	value := newObject((*_PyObject)(arg2))

	err := (*f)(obj, key, value)
	if err != nil {
		raise(err)
		return -1
	}

	return 0
}
