// Copyright 2011 Julian Phillips.  All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package py

// #include "utils.h"
//import "C"

import "unsafe"

type StartToken int

const (
	EvalInput StartToken = iota
	FileInput
	SingleInput
)

func RunFile(filename string, start StartToken, globals, locals Object) (Object, error) {
	name := _CString(filename)
	defer _free(unsafe.Pointer(name))

	mode := _CString("r")
	defer _free(unsafe.Pointer(mode))

	var token int32
	switch start {
	case EvalInput:
		token = _Py_eval_input
	case FileInput:
		token = _Py_file_input
	case SingleInput:
		token = _Py_single_input
	}

	file, err := _fopen(name, mode)
	if file == nil {
		return nil, err
	}
	defer _fclose(file)

	obj := _PyRun_FileExFlags(file, name, token, c(globals), c(locals), 0, nil)
	if obj == nil {
		return nil, exception()
	}

	return newObject(obj), nil
}
